"use strict";

var Promise = require("bluebird");

var AliasMarker = require('../../../main/js/map/marker/AliasMarker');
var AliasSurface = require('../../../main/js/map/surface/AliasSurface');
var AbstractCustomMap = require('../../../main/js/map/AbstractCustomMap');
var IdentifiedElement = require('../../../main/js/map/data/IdentifiedElement');

var chai = require('chai');
var assert = chai.assert;

var logger = require("../logger");

describe('AbstractCustomMap', function() {
  it("Constructor", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    var mockObject = new AbstractCustomMap(model, options);

    assert.ok(mockObject.getId());
    assert.ok(mockObject.getModel());
    assert.ok(mockObject.getSelectedLayoutOverlays());

    assert.ok(mockObject.pixelOrigin_);
    assert.ok(mockObject.pixelsPerLonDegree_);
    assert.ok(mockObject.pixelsPerLonRadian_);
    assert.ok(mockObject.zoomFactor);

    assert.equal(mockObject.getAliasInfoWindowById(-1), null);
    assert.equal(mockObject.getPointInfoWindowById(-1), null);
    assert.equal(mockObject.getReactionInfoWindowById(-1), null);

    assert.equal(logger.getWarnings().length, 0);
  });

  it("setupLayouts", function() {
    var model = helper.createModel();
    var layout = model.getLayouts()[0];
    var options = helper.createCustomMapOptions();

    var mockObject = new AbstractCustomMap(model, options);

    var gmap = helper.createGoogleMap();

    mockObject.setGoogleMap(gmap);

    mockObject.setupLayouts();
    assert.ok(mockObject.getGoogleMap().getMapTypeId().indexOf(layout.getId()) >= 0);
  });

  it("creatMapOptions", function() {
    var mockObject = helper.createCustomMap();
    assert.ok(mockObject.creatMapOptions());

  });

  it("createTypeOptions", function() {
    var layoutId = 40;
    var layoutName = 'some layout name';
    var layoutDirectory = 'dire45';

    var model = helper.createModel();
    var layoutDataObj = {
      idObject : layoutId,
      name : layoutName,
      directory : layoutDirectory
    };
    model.addLayout(layoutDataObj);

    var options = helper.createCustomMapOptions();

    var mockObject = new AbstractCustomMap(model, options);

    var result = mockObject.createTypeOptions(layoutDataObj);

    assert.ok(result);

  });

  it("setMaxZoomLevel", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setMaxZoom(4);

    var mockObject = new AbstractCustomMap(model, options);

    var gmap = helper.createGoogleMap();

    mockObject.setGoogleMap(gmap);
    mockObject.setMaxZoomLevel();

    assert.equal(model.getMaxZoom(), mockObject.getGoogleMap().getZoom());
  });

  it("getMouseLatLng", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setMaxZoom(4);

    var mockObject = new AbstractCustomMap(model, options);

    var gmap = new google.maps.Map(testDiv, {});
    gmap.setCenter(new google.maps.LatLng(1, 2));

    mockObject.setGoogleMap(gmap);

    var coords = mockObject.getMouseLatLng();

    assert.ok(coords);
    assert.ok(coords instanceof google.maps.LatLng);
  });

  it("fromPixelsToPoint 1", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setWidth(256);
    model.setHeight(256);
    model.setMinZoom(2);
    model.setMaxZoom(2);

    var mockObject = new AbstractCustomMap(model, options);

    var pointDist = {
      x : 120,
      y : 140
    };

    var normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 2);

    assert.ok(normalizedPoint);
    assert.equal(normalizedPoint.x, pointDist.x);
    assert.equal(normalizedPoint.y, pointDist.y);

  });

  it("fromPixelsToPoint 2", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setWidth(512);
    model.setHeight(512);
    model.setMinZoom(2);
    model.setMaxZoom(3);

    var mockObject = new AbstractCustomMap(model, options);

    var pointDist = {
      x : 120,
      y : 140
    };

    var normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 2);

    assert.ok(normalizedPoint);
    assert.equal(2 * pointDist.x, normalizedPoint.x);
    assert.equal(2 * pointDist.y, normalizedPoint.y);

    normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 3);

    assert.ok(normalizedPoint);
    assert.equal(pointDist.x, normalizedPoint.x);
    assert.equal(pointDist.y, normalizedPoint.y);

  });

  it("fromPixelsToPoint 3", function() {
    var model = helper.createModel();
    var options = helper.createCustomMapOptions();

    model.setWidth(512);
    model.setHeight(512);
    model.setMinZoom(0);
    model.setMaxZoom(3);

    var mockObject = new AbstractCustomMap(model, options);

    var pointDist = {
      x : 120,
      y : 140
    };

    var normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 2);

    assert.ok(normalizedPoint);
    assert.equal(0.5 * pointDist.x, normalizedPoint.x);
    assert.equal(0.5 * pointDist.y, normalizedPoint.y);

    normalizedPoint = mockObject.fromPixelsToPoint(pointDist, 3);

    assert.ok(normalizedPoint);
    assert.equal(0.25 * pointDist.x, normalizedPoint.x);
    assert.equal(0.25 * pointDist.y, normalizedPoint.y);

  });

  it("registerMapClickEvents", function() {
    var mockObject = helper.createAbstractCustomMap();

    var gmap = new google.maps.Map(testDiv, {});

    mockObject.setGoogleMap(gmap);
    // mock function to not put error log on console
    mockObject.getTopMap = function() {
      return null;
    };

    mockObject.registerMapClickEvents();
  });

  it("latLngToTile", function() {
    var mockObject = helper.createAbstractCustomMap();

    var latLng = new google.maps.LatLng(0, 1);

    var tileCoordinates = mockObject.latLngToTile(latLng, 3);
    assert.ok(tileCoordinates);
    assert.ok(tileCoordinates instanceof google.maps.Point);

  });

  it("_turnOnOffDrawing", function() {
    var mockObject = helper.createAbstractCustomMap();

    var gmap = new google.maps.Map(testDiv, {});

    mockObject.setGoogleMap(gmap);

    // mock function to not put error log on console
    mockObject.getTopMap = function() {
      return null;
    };

    assert.equal(mockObject.isDrawingOn(), false);
    mockObject._turnOnOffDrawing();
    assert.ok(mockObject.isDrawingOn());
    mockObject._turnOnOffDrawing();
    assert.equal(mockObject.isDrawingOn(), false);
  });

  it("areaToString", function() {
    var mockObject = helper.createAbstractCustomMap();

    var triangleCoords = [ new google.maps.LatLng(25.774, -80.190), new google.maps.LatLng(18.466, -66.118),
        new google.maps.LatLng(32.321, -64.757), new google.maps.LatLng(25.774, -80.190), ];

    // Construct the polygon.
    var bermudaTriangle = new google.maps.Polygon({
      paths : triangleCoords,
      strokeColor : '#FF0000',
      strokeOpacity : 0.8,
      strokeWeight : 2,
      fillColor : '#FF0000',
      fillOpacity : 0.35
    });

    assert.ok(mockObject.areaToString(bermudaTriangle));

  });

  it("Point - Lat,Lng conversion", function() {

    var mockObject = helper.createAbstractCustomMap();

    var x = 19920;
    var y = 11040;
    var point = new google.maps.Point(x, y);

    var coord = mockObject.fromPointToLatLng(point);

    var point2 = mockObject.fromLatLngToPoint(coord);

    assert.closeTo(point2.x, point.x, 0.5, "X coordinate is invalid after transformation");
    assert.closeTo(point2.y, point.y, 0.5, "Y coordinate is invalid after transformation");

  });

  it("_openInfoWindowForReaction", function() {
    var mockObject = helper.createAbstractCustomMap();

    var reaction = helper.createReaction(mockObject);

    // mock the behaviour of the map
    mockObject.getTopMap = function() {
      return mockObject;
    };
    mockObject.getOverlayDataForReaction = function() {
      return Promise.resolve([]);
    };

    mockObject.getModel().addReaction(reaction);

    assert.equal(null, mockObject.getReactionInfoWindowById(reaction.getId()));

    return mockObject._openInfoWindowForReaction(reaction).then(function() {
      assert.ok(mockObject.getReactionInfoWindowById(reaction.getId()));
    });

  });

  it("getDebug", function() {
    var map = helper.createAbstractCustomMap();
    map.setDebug(true);
    assert.ok(map.isDebug());
  });

  it("fitBounds", function() {
    var map = helper.createCustomMap();
    var alias = helper.createAlias(map);
    var ie = new IdentifiedElement(alias);
    var marker = new AliasMarker({
      map : map,
      element : ie
    });
    var surface = new AliasSurface({
      map : map,
      element : ie,
      gmapObj : new google.maps.Rectangle({
        map : map.getGoogleMap(),
        bounds : new google.maps.LatLngBounds(new google.maps.LatLng(0, 0), new google.maps.LatLng(0.5, 1)),
      })
    });

    var markers = [ marker, surface ];
    return marker.init().then(function() {

      var center = map.getGoogleMap().getCenter();
      map.fitBounds(markers);
      var center2 = map.getGoogleMap().getCenter();
      assert.ok(center.lat() !== center2.lat() || center.lng() !== center2.lng());
    });
  });

});
